package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.GroupDTO;
import cn.rankin.common.utils.dto.product.TagDTO;
import cn.rankin.common.utils.dto.search.GroupSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.productservice.entity.Group;
import cn.rankin.productservice.entity.Tag;
import cn.rankin.productservice.service.GroupService;
import cn.rankin.productservice.service.TagService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "group")
public class GroupController {

    @Autowired
    private GroupService groupService;

    @Autowired
    private TagService tagService;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Group>> search(GroupSearchDTO searchDTO) {
        Group group = new Group();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            group.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            group.setName("%" + name + "%");
        }

        // others
        group.setStatus(searchDTO.getStatus());
        group.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return groupService.search(group, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Group> create(@Valid @RequestBody GroupDTO groupDTO) {

        APIResult<Group> result = groupService.create(groupDTO);
        if (!result.getSuccess()) {
            return result;
        }

        Group group = result.getData();

        List<TagDTO> tagDTOList = groupDTO.getTagList();
        if (!CollectionUtils.isEmpty(tagDTOList)) {
            APIResult<List<Tag>> tagResult = tagService.updateGroupRelation(group.getId(), tagDTOList);
            group.setTagList(tagResult.getData());
        }

        return result;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Group> update(@Valid @RequestBody GroupDTO groupDTO) {
        APIResult<Group> result = groupService.update(groupDTO);
        if (!result.getSuccess()) {
            return result;
        }

        Group group = result.getData();

        List<TagDTO> tagDTOList = groupDTO.getTagList();
        if (tagDTOList != null) {
            APIResult<List<Tag>> tagResult = tagService.updateGroupRelation(group.getId(), tagDTOList);
            if (!tagResult.getSuccess()) {
                return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
            }
            group.setTagList(tagResult.getData());
        }

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String id) {
        APIResult<Boolean> groupResult = groupService.delete(id);
        if (!groupResult.getSuccess()) {
            return groupResult;
        }

        // 同时删除下属所有tag
        return tagService.deleteByGroupId(id);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Group> getGroup(@PathVariable("id") String id) {
        APIResult<Group> result = groupService.findNormalById(id);
        Group group = result.getData();
        if (!result.getSuccess() || group == null) {
            return result;
        }

        APIResult<List<Tag>> tagResult = tagService.findNormalByGroupId(id);
        if (!tagResult.getSuccess()) {
            return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
        }

        group.setTagList(tagResult.getData());
        return result;

    }

}
