package cn.rankin.productservice.controller;

import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.ItemDTO;
import cn.rankin.common.utils.dto.product.ItemPriceDTO;
import cn.rankin.common.utils.dto.search.ItemSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.productservice.entity.Item;
import cn.rankin.productservice.entity.ItemPrice;
import cn.rankin.productservice.entity.Tag;
import cn.rankin.productservice.proxy.ItemServiceProxy;
import cn.rankin.productservice.service.ItemService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "item")
public class ItemController {

    @Autowired
    private ItemService itemService;

    @Autowired
    private ItemServiceProxy itemServiceProxy;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public APIResult<Page<Item>> search(ItemSearchDTO searchDTO) {
        Item item = new Item();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            item.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            item.setName("%" + name + "%");
        }

        // others
        item.setStatus(searchDTO.getStatus());
        item.setType(searchDTO.getType());
        item.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return itemService.search(item, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Item> create(@RequestBody ItemDTO itemDTO) {
        String code = itemDTO.getCode();
        if (StringUtils.isEmpty(code) || itemDTO.getMerchantId() == null) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        // 保存商品信息
        APIResult<Item> result = itemService.create(itemDTO);
        Item item = result.getData();
        if (!result.getSuccess()) {
            return result;
        }

        String itemId = item.getId();

        // 保存标签关系
        List<String> tagIdList = itemDTO.getTagList();
        if (tagIdList != null) {
            APIResult<List<Tag>> tagResult = itemServiceProxy.updateTagRelation(itemId, tagIdList);
            if (!tagResult.getSuccess()) {
                // 显示回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
            }
            item.setTagList(tagResult.getData());
        }

        // 保存价格
        List<ItemPriceDTO> itemPriceDTOList = itemDTO.getPriceList();
        if (itemPriceDTOList != null) {
            List<ItemPrice> priceList = itemServiceProxy.updatePrice(itemId, itemPriceDTOList).getData();
            item.setPriceList(priceList);
        }

        return result;
    }

    @Transactional
    @RequestMapping(method = RequestMethod.PUT)
    public APIResult<Item> update(@RequestBody ItemDTO itemDTO) {
        if (itemDTO.getId() == null) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        // 保存商品信息
        APIResult<Item> result = itemService.update(itemDTO);
        Item item = result.getData();
        if (!result.getSuccess()) {
            return result;
        }

        String itemId = item.getId();
        // 保存标签关系
        List<String> tagIdList = itemDTO.getTagList();
        if (tagIdList != null) {
            APIResult<List<Tag>> tagResult = itemServiceProxy.updateTagRelation(itemId, tagIdList);
            if (!tagResult.getSuccess()) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return APIResult.error(new BaseCode(tagResult.getCode(), tagResult.getMessage()));
            }
            item.setTagList(tagResult.getData());
        }

        // 保存价格
        List<ItemPriceDTO> itemPriceDTOList = itemDTO.getPriceList();
        if (itemPriceDTOList != null) {
            List<ItemPrice> priceList = itemServiceProxy.updatePrice(itemId, itemPriceDTOList).getData();
            item.setPriceList(priceList);
        }

        return result;
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@PathVariable("id") String itemId) {
        if (itemId == null) {
            return APIResult.error(APICode.PARAMETER_ERROR);
        }

        return itemService.delete(itemId);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public APIResult<Item> getItem(@PathVariable("id") String Id) {
        Item item = itemServiceProxy.loadItem(Id);
        return APIResult.ok(item);
    }
}
