package cn.rankin.productservice.proxy;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.dto.product.ItemPriceDTO;
import cn.rankin.common.utils.enums.EnumUtil;
import cn.rankin.common.utils.enums.ItemTypeEnum;
import cn.rankin.productservice.entity.*;
import cn.rankin.productservice.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

@Service
public class ItemServiceProxy {

    @Autowired
    private ItemService itemService;

    @Autowired
    private ItemPriceService itemPriceService;

    @Autowired
    private ItemTagRelationService tagRelationService;

    @Autowired
    private TagService tagService;

    @Autowired
    private CourseServiceProxy courseServiceProxy;

    @Autowired
    private SupportServiceProxy supportServiceProxy;

    public Item loadItem(String itemId) {
        Item item = load(itemId);
        if (item != null) {
            List<ItemPrice> priceList = itemPriceService.getItemPrice(itemId);
            item.setPriceList(priceList);
        }
        return item;
    }

    public Item load(String itemId) {
        Item item = itemService.findOne(itemId);
        if (item == null) {
            return null;
        }

        String subId = item.getSubId();
        ItemTypeEnum itemType = item.getType();

        if (ItemTypeEnum.COURSE.equals(itemType)) {
            Course course = courseServiceProxy.getCourse(subId);
            item.setCourse(course);
        }else if (ItemTypeEnum.SUPPORT.equals(itemType)) {
            Support support = supportServiceProxy.getSupport(subId);
            item.setSupport(support);
        }

        List<Tag> tagList = getTagList(itemId);
        item.setTagList(tagList);

        return item;
    }

    public Item loadItem(String itemId, String packageId) {
        Item item = load(itemId);
        if (item != null) {
            List<ItemPrice> priceList = itemPriceService.getItemPrice(itemId, packageId);
            item.setPriceList(priceList);
        }
        return item;
    }

    public List<Tag> getTagList(String itemId) {
        List<ItemTagRelation> itemTagRelationList = tagRelationService.findAllByItemId(itemId);
        if (CollectionUtils.isEmpty(itemTagRelationList)) {
            return new ArrayList<>();
        }

        List<String> tagIdList = new ArrayList<>();
        itemTagRelationList.forEach( t -> tagIdList.add(t.getTagId()));

        APIResult<List<Tag>> tagResult = tagService.findByIds(tagIdList);
        return tagResult.getData();
    }

    @Transactional
    public APIResult<List<ItemPrice>> updatePrice(String itemId, List<ItemPriceDTO> priceDTOList) {
        List<ItemPrice> priceList = new ArrayList<>();
        if (priceDTOList == null) {
            return APIResult.ok();
        }

        if (priceDTOList.size() == 0) {
            itemPriceService.deleteByItemId(itemId);
        }else {
            priceList = itemPriceService.update(itemId, priceDTOList).getData();
        }

        return APIResult.ok(priceList);
    }

    @Transactional
    public APIResult<List<Tag>> updateTagRelation(String itemId, List<String> tagIdList) {
        List<Tag> tagList = new ArrayList<>();
        if (tagIdList == null) {
            return APIResult.ok();
        }

        if (tagIdList.size() == 0) {
            tagRelationService.deleteByItemId(itemId);
        }else {
            APIResult<List<Tag>> tagResult = tagService.findByIds(tagIdList);
            tagList = tagResult.getData();
            if (tagList == null || tagList.size() != tagIdList.size()) {
                return APIResult.error(APICode.error("包含错误的标签"));
            }

            Item item = itemService.findById(itemId).getData();
            if (item == null) {
                return APIResult.error(APICode.error("商品不存在"));
            }

            String merchantId = item.getMerchantId();
            ItemTypeEnum itemType = item.getType();
            for (int i = 0; i < tagList.size(); i++) {
                Tag tag = tagList.get(i);
                if (!tag.getMerchantId().equals(merchantId) || !EnumUtil.compare(itemType, tag.getType())) {
                    return APIResult.error(APICode.error("包含错误的标签"));
                }
            }

            tagRelationService.updateByItem(itemId, tagIdList);
        }

        return APIResult.ok(tagList);
    }
}
