package cn.rankin.productservice.service;

import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.dto.product.ItemDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.ItemStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.productservice.entity.Item;
import cn.rankin.productservice.repository.*;
import cn.rankin.productservice.utils.DTOConverter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.*;

@Slf4j
@Service
public class ItemService {

    @Autowired
    private ItemRepository itemRepository;

    public boolean exists(Item item) {
        Long count = itemRepository.count(item);
        return count > 0L;
    }

    public APIResult<List<Item>> findAllBySubId(String courseId) {
        List<Item> itemList = itemRepository.findAllBySubId(courseId);
        return APIResult.ok(itemList);
    }

    public APIResult<List<Item>> findAllByIds(List<String> ids) {
        List<Item> itemList = itemRepository.findByIds(ids);
        return APIResult.ok(itemList);
    }

    public APIResult<List<Item>> findAllByIds(List<String> ids, ItemStatusEnum status) {
        List<Item> itemList = itemRepository.findAllByIdIn(ids, status);
        return APIResult.ok(itemList);
    }

    public APIResult<Item> findById(String itemId) {
        Item item = itemRepository.find(itemId);
        if (!ItemStatusEnum.DEL.equals(item.getStatus())) {
            return APIResult.ok(item);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    public Item findOne(String itemId) {
        return itemRepository.find(itemId);
    }

    public APIResult<Page<Item>> search(Item item, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = itemRepository.count(item);
        Page<Item> page = new Page(count, pageNo, pageSize);

        if (count == 0) {
            return APIResult.ok(page);
        }

        List<Item> itemList = itemRepository.find(item, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        page.setList(itemList);

        return APIResult.ok(page);
    }

    @Transactional
    public APIResult<Item> create(ItemDTO itemDTO) {
        // 检查course与merchantId的唯一性
        Item sample = new Item();
        sample.setSubId(itemDTO.getSubId());
        sample.setMerchantId(itemDTO.getMerchantId());
        if (exists(sample)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        // 保存item数据
        Item item = DTOConverter.convert(itemDTO);
        Item result = itemRepository.save(item);

        return APIResult.ok(result);
    }

    @Transactional
    public APIResult<Boolean> delete(String itemId) {
        Integer count = itemRepository.updateStatusById(itemId, ItemStatusEnum.DEL);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(APICode.NOT_EXISTS);
    }

    @Transactional
    public APIResult<Item> update(ItemDTO itemDTO) {
        // 检查course与merchantId是否存在
        Item sample = new Item();
        sample.setSubId(itemDTO.getSubId());
        sample.setMerchantId(itemDTO.getMerchantId());
        if (!exists(sample)) {
            return APIResult.error(APICode.ALREADY_EXISTS);
        }

        // 保存item数据
        Item item = DTOConverter.convert(itemDTO);
        Item result = itemRepository.update(item);

        return APIResult.ok(result);
    }
}
